### get data
library(GEOquery)
library(tidyverse)
gset <- getGEO("GSE25055", GSEMatrix =TRUE, AnnotGPL=TRUE)
exprSet=exprs(gset[[1]])
pdata=pData(gset[[1]])

gset_val <- getGEO("GSE25065", GSEMatrix =TRUE, AnnotGPL=TRUE,
                   destdir = "./verification" )
exprSet_val=exprs(gset_val[[1]])
pdata_val=pData(gset_val[[1]])

## WGCNA analysis

library(WGCNA)
exp0 <- exprSet
m.vars=apply(exp0,1,var)
expro.upper=exp0[which(m.vars>quantile(m.vars, probs = seq(0, 1, 0.25))[4]),]
dim(expro.upper)
datExpr=as.data.frame(t(expro.upper));
nGenes = ncol(datExpr)
nSamples = nrow(datExpr)
datExpr <- t(exprSet)

datExpr_all = t(exp0)
dim(datExpr)
rownames(datExpr)
nGenes = ncol(datExpr)
nSamples = nrow(datExpr)

gsg = goodSamplesGenes(datExpr, verbose = 3);
gsg$allOK
sampleTree = hclust(dist(datExpr), method = "average")
plot(sampleTree, main = "Sample clustering to detect outliers"
     , sub="", xlab="") + abline(h = 250, col = "red") 
save(datExpr, file = "FPKM-01-dataInput.RData")
dev.off()

enableWGCNAThreads(nThreads = 10)

# 通过对 power 的多次迭代，确定最佳 power
powers = c(c(1:10), seq(from = 12, to=20, by=2))

sft = pickSoftThreshold(datExpr, powerVector = powers, verbose = 5)
# 计算出来的最佳 β 存放在
sft$powerEstimate

# 画图
pdf("1Threshold_all.pdf",width = 10, height = 5)
par(mfrow = c(1,2))
cex1 = 0.9
plot(sft_all$fitIndices[,1], -sign(sft_all$fitIndices[,3])*sft_all$fitIndices[,2],
     xlab="Soft Threshold (power)",ylab="Scale Free Topology Model Fit,signed R^2",type="n",
     main = paste("Scale independence")) +
  text(sft_all$fitIndices[,1], -sign(sft_all$fitIndices[,3])*sft_all$fitIndices[,2],
       labels=powers,cex=cex1,col="red")+
  abline(h=0.90,col="red")
plot(sft_all$fitIndices[,1], sft_all$fitIndices[,5],
     xlab="Soft Threshold (power)",ylab="Mean Connectivity", type="n",
     main = paste("Mean connectivity")) +
  text(sft_all$fitIndices[,1], sft_all$fitIndices[,5], labels=powers, cex=cex1,col="red")
dev.off()

net = blockwiseModules(datExpr, power = sft$powerEstimate,
                       TOMType = "unsigned", minModuleSize = 30,
                       reassignThreshold = 0, mergeCutHeight = 0.25,
                       numericLabels = TRUE, pamRespectsDendro = FALSE,
                       saveTOMs = TRUE,
                       #saveTOMFileBase = "MyTOM",
                       verbose = 3)

mergedColors = labels2colors(net$colors)

pdf("2module.pdf",width = 10, height = 5)
plotDendroAndColors(net$dendrograms[[1]], mergedColors[net$blockGenes[[1]]], "Module colors",
                    dendroLabels = FALSE, hang = 0.03,
                    addGuide = TRUE, guideHang = 0.05)
dev.off()

moduleLabelsAutomatic = net$colors
moduleColorsAutomatic = labels2colors(moduleLabelsAutomatic)
moduleColorsWW = moduleColorsAutomatic
MEs0 = moduleEigengenes(datExpr, moduleColorsWW)$eigengenes
MEsWW = orderMEs(MEs0)
modTraitCor = cor(MEsWW, datTraits, use = "p")
colnames(MEsWW)

nGenes = ncol(datExpr)
nSamples = nrow(datExpr)
modlues=MEsWW

modTraitP = corPvalueStudent(modTraitCor, nSamples)
textMatrix = paste(signif(modTraitCor, 2), "\n(", signif(modTraitP, 1), ")", sep = "")
dim(textMatrix) = dim(modTraitCor)



pdf("4Module-trait.pdf",width = 8, height = 10)
labeledHeatmap(Matrix = modTraitCor, xLabels = colnames(datTraits), yLabels = names(MEsWW), cex.lab = 0.5,  yColorWidth=0.01, 
               xColorWidth = 0.03,
               ySymbols = colnames(modlues), colorLabels = FALSE, colors = blueWhiteRed(50), 
               textMatrix = textMatrix, setStdMargins = FALSE, cex.text = 0.5, zlim = c(-1,1)
               , main = paste("Module-trait relationships"))
dev.off()

# Prognostic analysis
log_rank_p <- apply(MEs0, 2, function(values1){
  group=ifelse(values1>median(values1),'high','low')
  kmfit2 <- survfit(my.surv~group,data=MEs0)
  #plot(kmfit2)
  group <- factor(group, levels = c("low", "high"))
  data.survdiff <- survdiff(my.surv ~ group)
  p.val = 1 - pchisq(data.survdiff$chisq, length(data.survdiff$n) - 1)
  HR = (data.survdiff$obs[2]/data.survdiff$exp[2])/(data.survdiff$obs[1]/data.survdiff$exp[1])
  up95 = exp(log(HR) + qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  low95 = exp(log(HR) - qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  all <- c(p.val, HR, up95, low95)}
)

log_rank_blurgene <- apply(bluesur, 1, function(values1){
  group=ifelse(values1>median(values1),'high','low')
  kmfit2 <- survfit(my.surv~group,data=bluesur)
  #plot(kmfit2)
  group <- factor(group, levels = c("low", "high"))
  data.survdiff <- survdiff(my.surv ~ group)
  p.val = 1 - pchisq(data.survdiff$chisq, length(data.survdiff$n) - 1)
  HR = (data.survdiff$obs[2]/data.survdiff$exp[2])/(data.survdiff$obs[1]/data.survdiff$exp[1])
  up95 = exp(log(HR) + qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  low95 = exp(log(HR) - qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  all <- c(p.val, HR, up95, low95)}
)

### enriment analysis
blue <- genemodule %>% filter(colors == "blue")
go <- enrichGO(blue$gene, OrgDb = "org.Hs.eg.db", ont="all", qvalueCutoff = 0.2, pAdjustMethod = "BH")
kk <- enrichKEGG(gene         = blue$gene,
                 organism     = 'hsa',   ##老鼠的话为mmu/rno
                 pvalueCutoff = 0.05, qvalueCutoff = 0.2, pAdjustMethod = "BH")

### Identification of hub genes involved in taxane-anthracycline resistance
box <- lapply(list(up=one, down=two), function(ii) {
  dd <- bluesen[ii,]
  dd$gene = rownames(dd)
  d2 <- gather(dd, sample, expr, -gene)
  d2$sensitive <- clinical[match(d2$sample, clinical$geo_accession),3]
  pvalues <- sapply(d2$gene, function(x) {
    res <- kruskal.test(expr ~ sensitive, data = subset(d2, gene == x))
    res[["p.value"]]# 计算P值
  })
  pv <- data.frame(gene = d2$gene, pvalue = pvalues)
  
  ## Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
  pv$sigcode <- cut(pv$pvalue, c(0, 0.001, 0.01, 0.05, 0.1, 1), 
                    labels=c('***', '**', '*', '.', ' '))
  
  ggplot(d2, aes(gene, expr, fill=sensitive)) + 
    geom_boxplot(position = position_dodge(0.8)) + 
    geom_text(aes(gene, y=max(d2$expr) * 1.1, 
                  label=pv$sigcode),
              data=pv, inherit.aes=F) + 
    scale_fill_manual(values = c("#80B1D3", "#FB8072")) +
    xlab(NULL)+ylab("FPKM expression")
})
plot_grid(plotlist=box, ncol=1)

ggsave("sentitive_kruskal.pdf", width = 12, height = 8)